#!/usr/bin/env python
#-*- coding:utf-8 -*-

# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html

from gimpfu import *
####################################################################################################################
################# START gegl_command by Kevin Payne & Claude Lion ##################################################
####################################################################################################################

from sys import platform
from ctypes import *

def load_library (library_name):
	if platform == "linux" or platform == "linux2":
		library_name = library_name + '.so.0'
	elif platform == "win32":
		from ctypes.util import find_library
		library_name = find_library (library_name + "-0")
	else:
		raise BaseException ("TODO")
	return CDLL (library_name)

gimpC = load_library ('libgimp-2.0')


def gegl_command(image, drawable, gegl_graph_string):
	
	class GeglBuffer(Structure):
		pass
	drawable_id = drawable.ID
	
	gegl = load_library ('libgegl-0.4')
	gegl.gegl_init (None, None)
	gimpC.gimp_drawable_get_shadow_buffer.restype = POINTER (GeglBuffer)
	gimpC.gimp_drawable_get_buffer.restype        = POINTER (GeglBuffer)

	x = c_int (); y = c_int (); w = c_int (); h = c_int ()
	non_empty,x,y,w,h = pdb.gimp_drawable_mask_intersect (drawable)
	args = [b"string", c_char_p (gegl_graph_string), c_void_p ()]
	
	if non_empty:
		source = gimpC.gimp_drawable_get_buffer (drawable_id)
		target = gimpC.gimp_drawable_get_shadow_buffer (drawable_id)
		gegl.gegl_render_op (source, target, "gegl:gegl", *args)
		gegl.gegl_buffer_flush (target)
		gimpC.gimp_drawable_merge_shadow (drawable_id, PushUndo = True)
		gimpC.gimp_drawable_update (drawable_id, x, y, w, h)
		gimpC.gimp_displays_flush ()
		
####################################################################################################################
################# END gegl_command #################################################################################
####################################################################################################################
 
 
def gegl_test(image, drawable):

	layer_copy = pdb.gimp_layer_new_from_drawable(drawable, image)
	pdb.gimp_image_insert_layer(image, layer_copy, None, 0)
	gegl_command(image, layer_copy,'mosaic')
	pdb.gimp_item_set_name(layer_copy, "Gegl Mosaic DIFFERENCE")
	pdb.gimp_layer_set_mode(layer_copy, 6)	# DIFFERENCE-LEGACY
	pdb.gimp_image_set_active_layer(image,drawable)
	pdb.plug_in_gmic_qt(image,drawable, 1, 0, "deform 50")
	merged=pdb.gimp_image_merge_down(image, pdb.gimp_image_get_layer_by_name(image, "Gegl Mosaic DIFFERENCE"), 0)
	gegl_command(image, merged,'tile-seamless')	


register(
	"python-fu-gegl-test",
	"Test",
	"Test",
	"MrQ",
	"GPL3",
	"2022",
	"GEGL Test...",
	"*",
	[
		(PF_IMAGE,    "image", "takes current image", None),
		(PF_DRAWABLE, "drawable", "Input layer", None),
	],
	[],
	gegl_test, menu="<Image>/GEGL")

main()    

